"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function convertToFluent(gen) {
    return {
        toArray: () => Array.from(gen),
        takeUntilStable: () => convertToFluent(takeUntilStable(gen)),
        first: (predicate) => first(predicate, gen)
    };
}
function memoize(func) {
    const cache = {};
    return (input) => {
        const cacheHit = cache[input];
        if (cacheHit) {
            return cacheHit;
        }
        const result = func(input);
        cache[input] = result;
        return result;
    };
}
exports.memoize = memoize;
function first(predicate, gen) {
    const next = gen.next();
    if (next.done === true) {
        return undefined;
    }
    if (predicate(next.value)) {
        return next.value;
    }
    return first(predicate, gen);
}
function keepApplying(initialValue, modifier) {
    function* helper(initialValue, modifier) {
        yield initialValue;
        yield* helper(modifier(initialValue), modifier);
    }
    return convertToFluent(helper(initialValue, modifier));
}
exports.keepApplying = keepApplying;
function* takeUntilStable(gen) {
    function* helper(gen, previousValue) {
        yield previousValue;
        const next = gen.next();
        if (!next.done && next.value !== previousValue) {
            yield* helper(gen, next.value);
        }
    }
    const next = gen.next();
    if (!next.done) {
        yield* helper(gen, next.value);
    }
}
function flatten(array) {
    return array.reduce((prev, cur) => [...prev, ...cur], []);
}
exports.flatten = flatten;
function flattenObject(array) {
    return array.reduce((prev, cur) => (Object.assign(Object.assign({}, prev), cur)));
}
exports.flattenObject = flattenObject;
function dedupe(array) {
    return array.filter((value, index, array) => index === array.indexOf(value));
}
exports.dedupe = dedupe;
