"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const functional_1 = require("./functional");
function getAliasForProject(project) {
    if (typeof project === "undefined") {
        project = process.cwd();
    }
    const rootProjectPath = resolveTsConfig(project);
    const alias = functional_1.flattenObject(getReferencedProjectsRecursive(rootProjectPath).map(getAliasFor));
    return convertSlashes(alias);
}
exports.getAliasForProject = getAliasForProject;
function toForwardSlashes(input) {
    return input.replace(/\\/g, "/");
}
function convertSlashes(alias) {
    return Object.keys(alias).reduce((prev, cur) => (Object.assign(Object.assign({}, prev), { [toForwardSlashes(cur)]: toForwardSlashes(alias[cur]) })), {});
}
function resolveTsConfig(p) {
    const isFile = fs.statSync(p).isFile();
    const configPath = isFile ? p : path.join(p, "tsconfig.json");
    return configPath;
}
const getReferencedProjectsRecursive = functional_1.memoize((tsConfigPath) => {
    const directReferencedProjects = getReferencedProjects(tsConfigPath);
    return [
        tsConfigPath,
        ...functional_1.dedupe(functional_1.flatten(directReferencedProjects.map(getReferencedProjectsRecursive)))
    ];
});
function getReferencedProjects(tsConfigPath) {
    var _a;
    const projectDir = path.dirname(tsConfigPath);
    const config = require(tsConfigPath);
    const references = (_a = config.references) === null || _a === void 0 ? void 0 : _a.map(o => o.path).map(p => path.join(projectDir, p)).map(resolveTsConfig);
    return (references !== null && references !== void 0 ? references : []);
}
function isPackageDir(dir) {
    try {
        return fs.statSync(path.join(dir, "package.json")).isFile();
    }
    catch (_a) {
        return false;
    }
}
function findPackageJsonDir(dir) {
    return functional_1.keepApplying(dir, path.dirname)
        .takeUntilStable()
        .first(isPackageDir);
}
function tryGetPackageInfo(dir) {
    try {
        const packageDir = findPackageJsonDir(dir);
        const packageJson = require(path.join(packageDir, "package.json"));
        return { name: packageJson.name, module: packageJson.module, main: packageJson.main, packageDir };
    }
    catch (_a) {
        return undefined;
    }
}
function getAliasFor(tsConfigPath) {
    const projectRootDir = path.dirname(tsConfigPath);
    const config = require(tsConfigPath);
    const hasExplicitOutDir = typeof config.compilerOptions.outDir !== "undefined";
    const rootDir = path.join(projectRootDir, config.compilerOptions.rootDir || "");
    const outDir = hasExplicitOutDir
        ? path.join(projectRootDir, config.compilerOptions.outDir)
        : rootDir;
    const packageInfo = tryGetPackageInfo(projectRootDir);
    const alias = {};
    if (typeof packageInfo !== "undefined") {
        const name = packageInfo.name;
        const main = path.join(packageInfo.packageDir, packageInfo.module || packageInfo.main || "./index.js");
        const importLib = outDir.replace(packageInfo.packageDir, name);
        const importSrc = rootDir;
        const sourceOfMain = main.replace(outDir, rootDir).replace(".js", "");
        alias[importLib] = importSrc;
        alias[`${name}$`] = sourceOfMain;
    }
    return alias;
}
